var fs=require('fs');
var conf = require('./conf');
var compression = require('compression');
var express = require('express');
var bodyParser = require('body-parser');
var app = express();
var version = process.argv[2];

conf.nodePath = __dirname + '/';
conf.rootPath = __dirname.split('/');
conf.rootPath.pop();
conf.rootPath = conf.rootPath.join('/') + '/';

var staticFiles = [
                    {"target":"templates"},
                    {"target":"css"},
                    {"target":"js"}
                  ];
var pages = [
              //web pages
              //home page 
              {"name":"index","path":"/"},
              {"name":"contact-us","path":"/contact-us"},

              //web services
             {"name":"submit-contact","method":"post"}

            ];

app.use(compression()); // compress all requests
app.use(bodyParser.json()); // parse application/json


//backend functions
var mysql      = require('mysql');
var sql = require('./helper/sql.js');
var route = require('./route');


mysqlConn = mysql.createPool(conf.db);

//common functions
var Handlebars = require('handlebars');
var site = require('./helper/site');
var validate = require('./helper/validate.js');
var common = require('./helper/common.js');


for(var k in staticFiles){
  var spath = staticFiles[k];
    app.use('/' + spath.target + '-' + version,express.static(conf.nodePath + spath.target));
}

//read all template files and keep it in global variable
var templates = {};

fs.readdir(conf.nodePath + 'templates/',function(err,files){
  for(var k in files){
    templates[files[k]] = fs.readFileSync(conf.nodePath + 'templates/' + files[k], "utf8");
  }
});


var global = {
    "handleBar":Handlebars,
    "mysql":mysql,
    "site":site,
    "validate":validate,
    "common":common,
    "sql":sql,
    "conf":conf,
    "templates":templates,
    "version":version,
    "mysqlConn":mysqlConn
  }


for(var k in pages){
    var page = pages[k];
    var ppath = '/' + page.name;
    var json = false;
    var method = '';

    if(page.path!=undefined){
      ppath = page.path;
    }

    if(page.json==true){
      json = true;
    }

    if(page.method!=undefined){
      method = page.method;
    }

    
    if(method=="post" || json==true){
      //include service file
      var rpage = require('./services/' + page.name + '.js');
    }else{
      //include page file
      var rpage = require('./pages/' + page.name + '.js');
    }

    //set in global variable
    global[ppath] = {"name":page.name,"page":rpage,"json":json,"method":method};

    if(page.method=="post"){
      app.post(ppath, function (req, res) {
        var routeClient = new route;
        var path = req.route.path;
        var view = global[path]["name"];
        req.query['format'] = 'json';
        routeClient.process(global, path, view, req, res);
      });
    }else{
      app.get(ppath, function (req, res) {
        var routeClient = new route;
        var path = req.route.path;
        var view = global[path]["name"];
        if(global[path]["json"]){
          req.query['format'] = 'json';
        }
        routeClient.process(global, path, view, req, res);
      });
    }
}

var server = app.listen(conf.port, function () {
var host = server.address().address;
var port = server.address().port;

  console.log('caratlane app listening at http://%s:%s', host, port);
});
